/**
** This software is distributed under the terms of the General Public License.  
** 
** Program : BuildIT!
** Version : 0.2
** Authors : Artur Prusakiewicz <prusak@mat.uni.torun.pl>
**	     Michal Switala <scisly@mat.uni.torun.pl>
** E-MAIL  : build@mat.uni.torun.pl
** WWW     : http://www.mat.uni.torun.pl/~scisly/buildit/
**           http://www.mat.uni.torun.pl/~prusak/buildit/
**/

#include "UserPrefs.h"

void updateSig(int tab[]) {}

UserPrefs::UserPrefs(QWidget *parent, const char* name, QString path):QWidget(parent, name, 0)
{
  configPath = path.copy();
  connect(this, SIGNAL(updateSig(int [])), parent, SLOT(updatePref(int [])));
}

void UserPrefs::show(int options[])
{
  for (int i = 0; i < PREFNUMBER; i++)
    prefsTab[i] = options[i];
  
  tab = new QTabDialog(0, "top-level");
  tab->setCaption("User's Preferences");
  
  // view section

  QFrame *gr1 = new QFrame(this);
  gr1->setGeometry(10,10,250,100);

  QLabel *toolL = new QLabel(gr1);
  toolL->setGeometry(20, 20, 50, 20);
  toolL->setText("ToolBar");
  QCheckBox *toolB = new QCheckBox(gr1);
  toolB->setText("visible");
  if (prefsTab[0])
    toolB->setChecked( TRUE );
  else
    toolB->setChecked( FALSE);
  toolB->setGeometry(80, 20, 60, 20);
  //not active yet
  toolB->setEnabled(FALSE);
  connect(toolB, SIGNAL(toggled(bool)), SLOT(toolChange(bool)));

  QLabel *listL = new QLabel(gr1);
  listL->setGeometry(20, 40, 50, 20);
  listL->setText("List");
  QCheckBox *listB = new QCheckBox(gr1);
  listB->setText("visible");
  if (prefsTab[1])
    listB->setChecked( TRUE );
  else
    listB->setChecked( FALSE);
  listB->setGeometry(80, 40, 60, 20);
  //not active yet
  listB->setEnabled(FALSE);
  connect(listB, SIGNAL(toggled(bool)), SLOT(listChange(bool)));

  QLabel *treeL = new QLabel(gr1);
  treeL->setGeometry(20, 80, 140, 20);
  treeL->setText("Component's Tree Style");

  QComboBox *stylelist = new QComboBox(gr1);
  stylelist->setGeometry(170, 80, 80, 20);
  stylelist->insertItem("Windows", 0);
  stylelist->insertItem("Motif", 1);
  stylelist->setCurrentItem(prefsTab[2]);
  connect(stylelist, SIGNAL(activated(int)), SLOT(treeChange(int)));

  tab->addTab(gr1, "&View");
  
  // source section
  QFrame *gr2 = new QFrame(this);
  gr2->setGeometry(10,10,250,100);
  
  QLabel *srcL = new QLabel(gr2);
  srcL->setGeometry(20, 20, 100, 20);
  srcL->setText("Event's Model");

  QComboBox *srcStyle = new QComboBox(gr2);
  srcStyle->setGeometry(110, 20, 120, 20);
  srcStyle->insertItem("One-For-All", 0);
  srcStyle->insertItem("One-For-Every", 1);
  srcStyle->setCurrentItem(prefsTab[3]);
  connect(srcStyle, SIGNAL(activated(int)), SLOT(eventModel(int))); 

  QPushButton *helpB = new QPushButton(gr2);
  helpB->setGeometry(20, 50, 70, 20);
  helpB->setText("Help");
  QLabel *helpL = new QLabel(gr2);
  helpL->setGeometry(100, 50, 100, 20);
  helpL->setText("for current model");

  connect(helpB,SIGNAL(clicked()),SLOT(eventHelp()));

  QLabel *comL = new QLabel(gr2);
  comL->setGeometry(20, 90, 80, 20);
  comL->setText("Add comments");
  QCheckBox *comB = new QCheckBox(gr2);
  comB->setGeometry(110, 90, 40, 20);
  comB->setText("yes");
  if (prefsTab[4])
      comB->setChecked( TRUE );
  else
      comB->setChecked( FALSE);
  connect(comB, SIGNAL(toggled(bool)), SLOT(commChange(bool)));

  tab->addTab(gr2, "&Source");

  tab->resize(300, 200);
  tab->setCancelButton();
  tab->setDefaultButton("Save Options");

  connect(tab,SIGNAL(defaultButtonPressed()),SLOT(saveConfig()));
  connect(tab,SIGNAL(applyButtonPressed()),SLOT(changeOptions()));
  connect(tab,SIGNAL(cancelButtonPressed()),SLOT(cancelIt()));

  tab->show();

}

void UserPrefs::loadConfig(int opt[])
{
    for (int i = 0; i < PREFNUMBER; i++)
	opt[i] = 0;
    QFile confile(configPath+".build/buildit.cnf");
    if (confile.open(IO_ReadOnly) == FALSE)
	{
	    opt[0] = 1;
	    opt[1] = 1;
	    opt[3] = 1;
	    opt[4] = 1;
	}
    else
	{
	    QTextStream t( &confile );
	    QString s;          
	    while ( !t.eof() ) 
		{
		    s = t.readLine(); 
		    if (!s.isEmpty())
			{
			    if ((s.contains("#")==0) && (s.contains("[")==0))
				{
				    if (s.contains("TOOLBAR"))
					if (s.contains("YES"))
					    opt[0] = 1;
				    if (s.contains("LIST"))
					if (s.contains("YES"))
					    opt[1] = 1;
				    if (s.contains("COMPONENTS'S TREE STYLE"))
					if (s.contains("MOTIF"))
					    opt[2] = 1;
				    if (s.contains("EVENTS STYLE"))
					if (s.contains("ONE-FOR-EVERY"))
					    opt[3] = 1;
				    if (s.contains("ADD COMMENTS"))
					if (s.contains("YES"))
					    opt[4] = 1;
				}
			}
		}
	    confile.close();
	}
}     


void UserPrefs::saveConfig()
{
    QFile confile(configPath+".build/buildit.cnf");
    if (confile.open(IO_WriteOnly) == FALSE)
	{
	    printf("UserPrefs::saveConfig => cannot open file=%s.build/buildit.cnf to write\n", configPath.data());
	    exit(0);
	}
    QString buf;
    buf = "# BuildIT! ver."+QString(VERSION)+"   config file\n\n\n";
    buf += "[View]\n";
    buf += "  [Visible]\n";

    buf += "   TOOLBAR = ";
    if (prefsTab[0]) 
	buf += "YES\n";
    else
	buf += "NO\n";
    buf += "   LIST = ";
    if (prefsTab[1]) 
	buf += "YES\n";
    else
	buf += "NO\n";
    buf += "  \n[Tree]\n";
    buf += "  COMPONENTS'S TREE STYLE = ";
    if (prefsTab[2]) 
	buf += "MOTIF\n";
    else
	buf += "WINDOWS\n";
    buf += "\n[Source]\n";
    buf += "EVENTS STYLE = ";
    if (prefsTab[3]) 
	buf += "ONE-FOR-EVERY\n";
    else
	buf += "ONE-FOR-ALL\n";
    buf += "ADD COMMENTS = ";
    if (prefsTab[4]) 
	buf += "YES\n";
    else
	buf += "NO\n";
    confile.writeBlock(buf, buf.length());
    confile.close();
}

void UserPrefs::changeOptions()
{
    emit updateSig(prefsTab);
}

void UserPrefs::cancelIt()
{

}

void UserPrefs::load()
{

}

void UserPrefs::eventHelp()
{
    QString text;
    if (prefsTab[3])
	{
	    text = "\"One-For-Every\"\n";
	    text += "In this model inner classes are used to serve events.\n";
	    text += "Every AWT object has his own class (or classes) which serves his event(s)\n";
	    text += "Example:\n";
	    text += "If you have: \"Canvas your_canvas\" defined  and want to serve events\n";
	    text += "mouseClicked and mouseEntered then will be generated something like that:\n";
	    text += " (...)\n";
	    text += " your_canvas.addMouseListener( new your_canvasL7() );\n";
	    text += " (..)\n";
	    text += " class your_canvas2L7 extends MouseAdapter {\n";
	    text += "   public void mouseClicked(MouseEvent e) {\n";
	    text += "     here you can write code which will be executed when you click\n";
	    text += "     on your_canvas\n";
	    text += "   }\n";
	    text += "   public void mouseEntered(MouseEvent e) {\n";
	    text += "   }\n";
	    text += " }\n";
	    QMessageBox::information(tab, "Event's Model Help", text.data(), QMessageBox::Ok);
	}
    else
	{ 
	    text = "\"One-For-All\"\n";
	    text += "In this model one event method serves all AWT objects which want to use this event\n";
	    text += "Example:\n";
	    text += "If you have: \"Canvas your_canvas\" and \"Button your_button\" defined and want to serve\n";
	    text += "event: mouseClicked for these objects, then will be generated something like that:\n";
	    text += " (...)\n";
	    text += " your_canvas.addMouseListener( this ); // \"this\" points at your project's class\n";
	    text += " your_button addMouseListener( this );\n";
	    text += " (..)\n";
            text += " public void mouseClicked(MouseEvent e) {\n";
	    text += "   here you can write code which will be executed when you click mouse\n";
	    text += " }\n";
	    text += " The \"problem\" is, that in above method you have to find out which object call this method\n";
	    text += " (your_canvas or your_button in this example)\n";
	    text += " Additional will be generated text for the rest of methods from MouseListener which have to be\n";
	    text += " in your source, because your project's class implements MouseListener\n";
	    QMessageBox::information(tab, "Event's Model Help", text.data(), QMessageBox::Ok);
	}

}

void UserPrefs::eventModel(int val)
{
    prefsTab[3] = val;
}

void UserPrefs::toolChange(bool val)
{
    if (val) 
	prefsTab[0] = 1;
    else
	prefsTab[0] = 0;
}

void UserPrefs::listChange(bool val)
{
    if (val) 
	prefsTab[1] = 1;
    else
	prefsTab[1] = 0;
}

void UserPrefs::commChange(bool val)
{
    if (val) 
	prefsTab[4] = 1;
    else
	prefsTab[4] = 0;
}


void UserPrefs::treeChange(int val)
{
    prefsTab[2] = val;
}
